# twbot.rbを用い、RSSを取得してTwitterに投稿するbotプログラムのサンプル

require "cgi"
require "open-uri"
require "twbot"

RSS_URL = "http://d.hatena.ne.jp/maraigue/rss"

class RSSPost < TwBot
  def load_data
    # RSSを取得する。
    buf = nil
    open(RSS_URL){ |file| buf = file.read }
    
    result = []
    @config["already_gotten"] ||= ""
    
    # 厳密にはREXMLを使うべきだが、ここでは簡略化している。
    # entryにはRSSでの各記事が格納される。
    buf.scan(/<item.*?>.*?<\/item>/m).each do |entry|
      # タイトル・URLを取得
      title = nil
      entry.scan(/<title>(.*?)<\/title>/){ |tmp| title = CGI.escapeHTML(tmp[0]) }
      link = nil
      entry.scan(/<link>(.*?)<\/link>/){ |tmp| link = CGI.escapeHTML(tmp[0]) }
      
      # すでに取得したことのあるデータとURLが一致していたら、
      # それ以降の（それよりも古い記事の）データはすべて無視する。
      break if link == @config["already_gotten"]
      
      # タイトル・URLが取得できていたら、投稿する発言のリストに追加する。
      # Twitterに投稿される順序はリストの順序であるが、RSSでは新しい記事ほど
      # 先に来るので、resultの後ろから順に投稿内容を追加している。
      result.unshift("#{link} #{title}") if title && link
    end
    
    # 取得したデータのうち、最も新しい記事のURLを保持しておく。
    unless result.empty?
      @config["already_gotten"] = result.last.split(/ /).first
    end
    
    # ログされるメッセージと結果を返す。
    @logmsg = "(#{result.size} post added)"
    result
  end
end

# プログラム起動時は以下のようにする
# ruby rsspost-sample.rb load        # RSSをロードして発言リストに追加する
# ruby rsspost-sample.rb post        # ロード済みの発言を1つ発言する
# ruby rsspost-sample.rb post post   # ロード済みの発言を2つ発言する
# ruby rsspost-sample.rb load post   # RSSをロードしたのち、発言を1つ発言する
SELFDIR = File.dirname(__FILE__)
ARGV.each do |mode|
  RSSPost.new mode, "#{SELFDIR}/config.yml", "#{SELFDIR}/error.log"
end
